VINFO_SHEET_NAME = "vInfo"
"""Name of the Excel sheet to look for VM data"""

STORAGE_COLUMN_NAME = "Provisioned"
"""Name of the column to look for VM storage capacity"""

MIB_TO_MB_CONVERSION_FACTOR = 1.04858
"""Conversion factor to convert MiB to MB"""

DEFAULT_OS_NAME = "Windows Server 2019 Datacenter"
"""Use this OS Name when no data is available to determine the OS"""



import logging

from rvtools2azmigrate.config import DEFAULT_OS_NAME, MIB_TO_MB_CONVERSION_FACTOR
from rvtools2azmigrate.azmigrate_vm import AzMigrateVM

log = logging.getLogger(__name__)


class RvToolsVM:
    """Class to represent a VM from the RVTools file"""

    def __init__(
        self,
        is_anonymized: bool,
        name: str,
        power_state: str,
        cores: int,
        memory: int,
        os_config: str,
        os_vmtools: str,
        storage_capacity: int,
        is_mib: bool,
        dns_name: str,
        uuid: str,
        firmware: str,
    ):
        """Init the RvToolsVM class

        Args:
            is_anonymized (bool): Does the VM needs needs to be anonymized?
            name (str): Name of the VM in RVTools report
            power_state (str): Power status of VM
            cores (int): Number of vCPU for the VM
            memory (int): Memory allocated to the VM
            os_config (str): OS name from the configuration
            os_vmtools (str): OS name from the VMware Tools
            storage_capacity (int): Storage capacity of the VM
            is_mib (bool): Is the storage in MiB ?
            dns_name (str): DNS name of the VM
            uuid (str): UUID of the VM
            firmware (str): Boot firmware of the VM
        """
        if is_anonymized:
            self.name = uuid
        else:
            self.name = name
        self.power_state = power_state
        self.cores = cores
        self.memory = memory
        if os_vmtools:
            self.os = os_vmtools
        elif os_config:
            self.os = os_config
        else:
            log.warning(f"OS not found for VM {self.name}")
            self.os = DEFAULT_OS_NAME
        if "64-bit" in self.os:
            self.architecture = "x64"
        elif "32-bit" in self.os:
            self.architecture = "x86"
        else:
            log.warning(f"Architecture not found for VM {self.name}")
            self.architecture = ""
        if is_mib:
            self.storage_capacity = int(storage_capacity / MIB_TO_MB_CONVERSION_FACTOR)
        self.storage_capacity_gb = int(storage_capacity / 1024)
        self.dns_name = dns_name
        self.uuid = uuid
        self.firmware = firmware

    def __repr__(self):
        return f"RvToolsVM({self.name}"

    def convert_to_az_migrate(self):
        """Convert the data to AzMigrateVM object"""
        return AzMigrateVM(
            server_name=self.name,
            cores=self.cores,
            memory=self.memory,
            os=self.os,
            architecture=self.architecture,
            boot_type="UEFI" if self.firmware == "efi" else "BIOS",
            disk1_size=self.storage_capacity_gb,
        )



"""AzMigrateVM class definition
"""

import logging

log = logging.getLogger(__name__)


class AzMigrateVM:
    """Class to represent a VM in Azure Migrate import format"""

    def __init__(
        self, server_name: str, cores: int, memory: int, os: str, architecture: str, boot_type: str, disk1_size: int
    ):
        """Init the AzMigrateVM class

        Args:
            server_name (str): Name of the server
            cores (int): Number of CPU cores for the server
            memory (int): Memory allocated to the server (MB)
            os (str): OS name
            architecture (str): Architecture of the OS
            boot_type (str): Boot type of the server
            disk1_size (int): Disk size of the server (GB)
        """
        self.server_name = server_name.replace(' ', '-').replace('_', '-').replace('.', '-').replace(',', '-')
        self.cores = cores
        self.memory = memory
        self.os = os
        self.architecture = architecture
        self.boot_type = boot_type
        self.disk1_size = disk1_size

    def __repr__(self):
        return f"AzMigrateVM({self.server_name})"

    def as_csv_row(self, forced_name: str = None) -> dict:
        """Represent the current object as a CSV row for Azure Migrate import format

        Args:
            forced_name (str): Force a specific name in the data (default: None)

        Returns:
            dict: a dict representation of CSV data to use in Azure Migrate import file.
        """
        csv_data = {}
        if forced_name:
            csv_data["*Server Name"] = forced_name
        else:
            csv_data["*Server Name"] = self.server_name
        csv_data["IP addresses"] = ""
        csv_data["*Cores"] = self.cores
        csv_data["*Memory (In MB)"] = self.memory
        csv_data["*OS name"] = self.os
        csv_data["OS version"] = ""
        csv_data["OS architecture"] = self.architecture
        csv_data["CPU utilization percentage"] = ""
        csv_data["Memory utilization percentage"] = ""
        csv_data["Network adapters"] = ""
        csv_data["Network In throughput"] = ""
        csv_data["Network Out throughput"] = ""
        csv_data["Boot type"] = self.boot_type
        csv_data["Number of disks"] = ""
        csv_data["Disk 1 size (In GB)"] = self.disk1_size
        csv_data["Disk 1 read throughput (MB per second)"] = ""
        csv_data["Disk 1 write throughput (MB per second)"] = ""
        csv_data["Disk 1 read ops (operations per second)"] = ""
        csv_data["Disk 1 write ops (operations per second)"] = ""
        csv_data["Disk 2 size (In GB)"] = ""
        csv_data["Disk 2 read throughput (MB per second)"] = ""
        csv_data["Disk 2 write throughput (MB per second)"] = ""
        csv_data["Disk 2 read ops (operations per second)"] = ""
        csv_data["Disk 2 write ops (operations per second)"] = ""
        return csv_data



"""Main module."""
import os
import logging
import csv
from openpyxl import load_workbook
from openpyxl.utils import get_column_letter

from rvtools2azmigrate.config import VINFO_SHEET_NAME, STORAGE_COLUMN_NAME
from rvtools2azmigrate.rvtools_vm import RvToolsVM

log = logging.getLogger(__name__)


def get_column_data_by_name(ws, column_name: str, default=None):
    """Get column data by name

    Args:
        ws (openpyxl.worksheet.worksheet.Worksheet): Worksheet
        column_name (str): Column name

    Returns:
        list: List of values or None if not found
    """
    for column_cell in ws.iter_cols(1, ws.max_column):  # iterate column cell
        if column_cell[0].value == column_name:  # check for your column
            return [data.value for data in column_cell[1:]]  # iterate your column
    log.warning(f"Column {column_name} not found")
    return None


def get_rvtools_vms(rvtools_data: dict, anonymized: bool, filter_off_vms: bool) -> list():
    """Get a list of RvToolsVM objects based on the RVTools data

    Args:
        rvtools_data (dict): RVTools data as a dict
        anonymized (bool): Anonymize the output data ?
        filter_off_vms (bool): Filter the powered-off VMs

    Returns:
        list(): List of RvToolsVM objects
    """
    rvtools_vms = []
    nb_vms = len(rvtools_data["vms_name"])
    vm_name_already_used = []
    for i in range(nb_vms):
        rv_vm = RvToolsVM(
            is_anonymized=anonymized,
            name=rvtools_data["vms_name"][i],
            power_state=rvtools_data["power_states"][i],
            cores=rvtools_data["cores"][i],
            memory=rvtools_data["memory"][i],
            os_config=rvtools_data["os_config"][i],
            os_vmtools=rvtools_data["os_vmtools"][i],
            storage_capacity=rvtools_data["storage_capacity"][i],
            is_mib=rvtools_data.get("is_mib", True),
            dns_name=rvtools_data["dns_name"][i],
            uuid=rvtools_data["uuid"][i],
            firmware=rvtools_data["firmware"][i],
        )
        # Filtering powered off VMs
        if rv_vm.power_state == "poweredOff" and filter_off_vms:
            _action = "Ignored"
        else:
            _action = "Processed"
            rvtools_vms.append(rv_vm)
        log.debug(f"{_action} VM {rv_vm.name} -> {rv_vm.power_state} ({i+1}/{nb_vms})")
    return rvtools_vms


def write_azmigrate_file(rvtools_vms: list, output: str):
    """Write Azure Migrate file

    Args:
        rvtools_vms (list): List of RvToolsVM
        output (str): Output file path
    """
    vm_name_already_used = []
    with open(output, "w", newline="") as f:
        # Build CSV headers based on data keys
        _first_vm = rvtools_vms[0].convert_to_az_migrate()
        writer = csv.DictWriter(f, fieldnames=_first_vm.as_csv_row().keys())
        writer.writeheader()
        # Append VM data
        for rv_vm in rvtools_vms:
            # Get the AZ migrate data from RVTools data
            vm_data = rv_vm.convert_to_az_migrate()
            # Check name uniqueness
            vm_name = vm_data.server_name
            if vm_name in vm_name_already_used:
                # Add an index to VM with similar naming
                vm_name = f"{vm_name}-{vm_name_already_used.count(vm_name)}"
            writer.writerow(vm_data.as_csv_row(forced_name=vm_name))
            # Save name occurrence (for uniqueness checks)
            vm_name_already_used.append(vm_data.server_name)


def parse_rvtools_data(rvtools: str) -> dict:
    """Parse the RVTools data to export VMs details

    Args:
        rvtools (str): Input file in RVTools format

    Returns:
        dict: RVTools data per column
    """
    # Open workbook and vInfo sheet
    wb = load_workbook(filename=rvtools, data_only=True)
    ws = wb[VINFO_SHEET_NAME]

    # Lookup for the storage column name based on MiB or MB unit
    is_mib = True
    storage_column_name = STORAGE_COLUMN_NAME + " MiB"
    storage_data = get_column_data_by_name(ws, column_name=storage_column_name)
    if not storage_data:
        is_mib = False
        storage_column_name = STORAGE_COLUMN_NAME + " MB"
        storage_data = get_column_data_by_name(ws, column_name=storage_column_name)
    log.info(f"Using the storage column name: {storage_column_name}")

    # Returns useful columns data
    return {
        "vms_name": get_column_data_by_name(ws, column_name="VM"),
        "power_states": get_column_data_by_name(ws, column_name="Powerstate"),
        "cores": get_column_data_by_name(ws, column_name="CPUs"),
        "memory": get_column_data_by_name(ws, column_name="Memory"),
        "os_config": get_column_data_by_name(ws, column_name="OS according to the configuration file"),
        "os_vmtools": get_column_data_by_name(ws, column_name="OS according to the VMware Tools"),
        "storage_capacity": get_column_data_by_name(ws, column_name=storage_column_name),
        "dns_name": get_column_data_by_name(ws, column_name="DNS Name"),
        "uuid": get_column_data_by_name(ws, column_name="VM UUID"),
        "firmware": get_column_data_by_name(ws, column_name="Firmware"),
        "is_mib": is_mib,
    }


def convert_rvtools_to_azmigrate(rvtools: str, output: str, anonymized: bool, filter_off_vms: bool):
    """Convert RVTools file to Azure Migrate format

    Args:
        rvtools (str): Input file in RVTools format
        output (str): Output file in Azure Migrate CSV format
        anonymized (bool): Anonymize VM names
        filter_off_vms (bool): Filter the powered-off VMs
    """
    # Read data from RVTools file
    rvtools_data = parse_rvtools_data(rvtools)
    log.info(f"We have found {len(rvtools_data['vms_name'])} VMs in the file {rvtools}")

    # Create a list of VMs
    rvtools_vms = get_rvtools_vms(rvtools_data, anonymized, filter_off_vms)
    log.info(f"VM data set from RVtools was built successfully")

    # Export VMs to an AZ migrate CSV file
    write_azmigrate_file(rvtools_vms, output)
    log.info(f"File {output} created successfully with {len(rvtools_vms)} VMs")
    return 0



"""Console script for rvtools2azmigrate."""
import sys
import click
import logging
from rich.logging import RichHandler

from rvtools2azmigrate.convert import convert_rvtools_to_azmigrate


LOG_FORMAT = "%(message)s"
log = logging.getLogger(__name__)


@click.group()
@click.option("--debug", default=False, is_flag=True)
def cli(debug):
    # Configure logging level and formating
    logging.basicConfig(
        level=logging.DEBUG if debug else logging.INFO,
        format=LOG_FORMAT,
        datefmt="[%X]",
        handlers=[RichHandler()],
    )
    click.echo("RVTools to Azure Migrate converter")
    log.debug(f"Debug mode is {'on' if debug else 'off'}")
    return 0


@cli.command()  # @cli, not @click!
@click.option(
    "-i",
    "--rvtools",
    help="RVTools input file",
    required=True,
    type=click.Path(exists=True),
)
@click.option("-o", "--output", help="Ouptut file", required=True, type=click.Path(exists=False))
@click.option(
    "--anonymized",
    default=False,
    is_flag=True,
    help="Anonymize the output file by replacing VM names with UUIDs",
)
@click.option(
    "--filter-off-vms",
    default=False,
    is_flag=True,
    help="Filter the powered-off VMs",
)
def convert(rvtools: str, output: str, anonymized: bool, filter_off_vms: bool):
    """Convert RVTools file to Azure Migrate format"""
    log.info("Starting RVTools file conversion to Azure Migrate format...")
    log.debug(f"Input file: {click.format_filename(rvtools)}")
    log.debug(f"Output file: {click.format_filename(output)}")
    log.debug(f"Anonymized: {anonymized}")
    log.debug(f"Filter powered-off VMs: {filter_off_vms}")
    convert_rvtools_to_azmigrate(rvtools=rvtools, output=output, anonymized=anonymized, filter_off_vms=filter_off_vms)
    return 0


if __name__ == "__main__":
    sys.exit(cli())  # pragma: no cover



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

requirements = [
    "coloredlogs",
    "Click",
    "openpyxl",
    "rich",
]

setup_requirements = []

test_requirements = []

description = "Convert an RVTools export to an Azure Migrate CSV inventory file"
# In case of long description
# description +=

setup(
    author="Ludovic Rivallain",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    description=description,
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": [
            "rvtools2azmigrate=rvtools2azmigrate.cli:cli",
            "rv2azm=rvtools2azmigrate.cli:cli",
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="rvtools2azmigrate",
    name="rvtools2azmigrate",
    packages=find_packages(include=["rvtools2azmigrate", "rvtools2azmigrate.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/lrivallain/rvtools2azmigrate",
    version="0.1.3",
    zip_safe=False,
)
